/******************************************
  PURPOSE:  Code for Zippy the Fan
   AUTHOR:  Tyler R. Oswald
     DATE:   Novemeber 2017
*******************************************/

// Including the libraries for the IR sensor and ESCs
#include <IRremote.h>
#include <Servo.h>

// Defining the IR sensor pin, ESC pins, and battery level pin
const int RecieverPin = 11;
const int ESC1Pin = 5;
const int ESC2Pin = 6;
const int ESC3Pin = 9;
const int ESC4Pin = 10;
const int BatteryPin = A0;

// Defining the variables which are inputs to the write() method in the Servo library
// Note that the input to this method is a value between 0 and 360 as if controlling a servo.
// For the SimonK ESCs used, the minimum value which starts the motor is 50.
const int MaxSpeed = 130;                   // This is the max speed the motors will be allowed to achieve.
int SpeedIncrement = 2;               // This is the desired speed increment (SpeedIncrement >= 2).
int StopValue = 50 - SpeedIncrement;  // This is the stop value which is one SpeedIncrement less than 50.
int ForwardSpeed = StopValue;   
int ReverseSpeed = StopValue;               // These variables will control the direction of the bot
int LeftSpeed = StopValue;                  // by adjusting the appropriate motor speeds.
int RightSpeed = StopValue;                 // Initially, the are all set to the stop value.

// Defining the variables used to read the lipo battery level if desired
int AnalogIn;                     // This variable reads in a value from 0 to 1023 from BatteryPin. 
float LipoLevel;                  // This variable converts the AnalogIn value to a percentage between 0% and 100%.
unsigned long CurrentTime;        
long PreviousTime;                // These variables establish a timer so that the battery level is
const int TimeInterval = 2500;    // displayed at desired time intervals specified in milliseconds.

// Calling the necessary constructors
IRrecv IRSignal(RecieverPin);   // Instantiating the IR sensor
decode_results SignalResults;   // Instantiating the signals recieved by the sensor
Servo ESC1, ESC2, ESC3, ESC4;   // Instantiating the ESCs

// The startup loop
void setup() {
  for (int i=2; i<=13; i++) {digitalWrite(i,LOW);}  // Ensures all digital pins are set LOW before waiting one second
  delay(1000);
  
  Serial.begin(9600);                               // Starts the serial communication
  
  IRSignal.enableIRIn();                            // Enabling the IR sensor
                              
  ESC1.attach(ESC1Pin); ESC1.write(40);             // Linking the ESC signal wires to the appropriate pins
  ESC2.attach(ESC2Pin); ESC2.write(40);             // and arming the ESCs by writing a value of 40 to each ESC
  ESC3.attach(ESC3Pin); ESC3.write(40);
  ESC4.attach(ESC4Pin); ESC4.write(40);
}

// The main loop which will run repeatedly
void loop() {
  
  if (IRSignal.decode(&SignalResults)) {

    // If the center navigation button on the remote is pressed,
    // this code stops all motors and resets all speed variables to the stop value.
    if (SignalResults.value == 2331063592) {
      ESC1.write(StopValue); ESC2.write(StopValue);
      ESC3.write(StopValue); ESC4.write(StopValue);
      ForwardSpeed = StopValue; ReverseSpeed = StopValue;
      LeftSpeed = StopValue; RightSpeed = StopValue; 
    }
    
    // If a numeric button 1, 2, or 3 is pressed, the speed increment is set to 2, 5, or 10 respectively.
    // These buttons will also stop all motors and reset all speed variables to the stop value.
    if (SignalResults.value == 3778927144) {
      SpeedIncrement = 2;
      StopValue = 50 - SpeedIncrement;
      ESC1.write(StopValue); ESC2.write(StopValue);
      ESC3.write(StopValue); ESC4.write(StopValue);
      ForwardSpeed = StopValue; ReverseSpeed = StopValue;
      LeftSpeed = StopValue; RightSpeed = StopValue;
    }
    if (SignalResults.value == 2908251746) {
      SpeedIncrement = 5;
      StopValue = 50 - SpeedIncrement;
      ESC1.write(StopValue); ESC2.write(StopValue);
      ESC3.write(StopValue); ESC4.write(StopValue);
      ForwardSpeed = StopValue; ReverseSpeed = StopValue;
      LeftSpeed = StopValue; RightSpeed = StopValue;
    }
    if (SignalResults.value == 657459652) {
      SpeedIncrement = 10;
      StopValue = 50 - SpeedIncrement;
      ESC1.write(StopValue); ESC2.write(StopValue);
      ESC3.write(StopValue); ESC4.write(StopValue);
      ForwardSpeed = StopValue; ReverseSpeed = StopValue;
      LeftSpeed = StopValue; RightSpeed = StopValue;
    }

    // If the up navigation button on the remote is pressed,
    // ForwardSpeed increases IF ReverseSpeed is equal to the stop value.
    // If ReverseSpeed is greater than the stop value, it is decreased until the stop value is reached.
    // Beforehand, LeftSpeed and RightSpeed are ensured to be reset to the stop value.
    if (SignalResults.value == 3261853764) {
      if (LeftSpeed > StopValue | RightSpeed > StopValue) {
        LeftSpeed = StopValue; RightSpeed = StopValue;          // Resetting LeftSpeed and RightSpeed to the stop value
        ESC1.write(StopValue); ESC2.write(StopValue);           // and stopping the motors before proceeding
        ESC3.write(StopValue); ESC4.write(StopValue);
        ForwardSpeed = StopValue - SpeedIncrement;              // This ensures ESC1 and ESC2 are stopped until the
      }                                                         // up navigation button is pressed again.
      if (ReverseSpeed > StopValue) {
        ReverseSpeed = ReverseSpeed - SpeedIncrement;           // If ReverseSpeed is greater than StopValue, reduce speed
        ESC3.write(ReverseSpeed); ESC4.write(ReverseSpeed);     // to ESC3 and ESC4, otherwise, increase speed to ESC1 and ESC2
      }                                                         // in order to move the bot forward
      else {
        ForwardSpeed = ForwardSpeed + SpeedIncrement;
        if (ForwardSpeed > MaxSpeed) {ForwardSpeed = MaxSpeed;} // The speed can be increased to the maximum allowable speed.
        ESC1.write(ForwardSpeed); ESC2.write(ForwardSpeed);
      }
    }

    // If the down navigation button on the remote is pressed,
    // ReverseSpeed increases IF ForwardSpeed is equal to the stop value.
    // If ForwardSpeed is greater than the stop value, it is decreased until the stop value is reached.
    // Beforehand, LeftSpeed and RightSpeed are ensured to be reset to the stop value.
    if (SignalResults.value == 3305092678) {
      if (LeftSpeed > StopValue | RightSpeed > StopValue) {
        LeftSpeed = StopValue; RightSpeed = StopValue;          // Resetting LeftSpeed and RightSpeed to the stop value
        ESC1.write(StopValue); ESC2.write(StopValue);           // and stopping the motors before proceeding
        ESC3.write(StopValue); ESC4.write(StopValue);
        ReverseSpeed = StopValue - SpeedIncrement;              // This ensures ESC3 and ESC4 are stopped until the
      }                                                         // down navigation button is pressed again.
      if (ForwardSpeed > StopValue) {
        ForwardSpeed = ForwardSpeed - SpeedIncrement;           // If ForwardSpeed is greater than StopValue, reduce speed
        ESC1.write(ForwardSpeed); ESC2.write(ForwardSpeed);     // to ESC1 and ESC2, otherwise, increase speed to ESC3 and ESC4
      }                                                         // in order to move the bot in reverse
      else {
        ReverseSpeed = ReverseSpeed + SpeedIncrement;
        if (ReverseSpeed > MaxSpeed) {ReverseSpeed = MaxSpeed;}
        ESC3.write(ReverseSpeed); ESC4.write(ReverseSpeed);     // The speed can be increased to the maximum allowable speed.
      }
    }

    // If the left navigation button on the remote is pressed,
    // LeftSpeed increases IF RightSpeed is equal to the stop value.
    // If RightSpeed is greater than the stop value, it is decreased until the stop value is reached.
    // Beforehand, ForwardSpeed and ReverseSpeed are ensured to be reset to the stop value.
    if (SignalResults.value == 1972149634) {
      if (ForwardSpeed > StopValue | ReverseSpeed > StopValue) {
        ForwardSpeed = StopValue; ReverseSpeed = StopValue;       // Resetting ForwardSpeed and ReverseSpeed to the stop value
        ESC1.write(StopValue); ESC2.write(StopValue);             // and stopping the motors before proceeding
        ESC3.write(StopValue); ESC4.write(StopValue);
        LeftSpeed = StopValue - SpeedIncrement;                   // This ensures ESC1 and ESC4 are stopped until the
      }                                                           // left navigation button is pressed again.
      if (RightSpeed > StopValue) {
        RightSpeed = RightSpeed - SpeedIncrement;                 // If RightSpeed is greater than StopValue, reduce speed
        ESC2.write(RightSpeed); ESC3.write(RightSpeed);           // to ESC2 and ESC3, otherwise, increase speed to ESC1 and ESC4
      }                                                           // in order to rotate the bot counterclockwise
      else {
        LeftSpeed = LeftSpeed + SpeedIncrement;
        if (LeftSpeed > MaxSpeed) {LeftSpeed = MaxSpeed;}         // The speed can be increased to the maximum allowable speed.
        ESC1.write(LeftSpeed); ESC4.write(LeftSpeed);
      }      
    }

    // If the right naviagtion on the remote is pressed,
    // RightSpeed increases IF LeftSpeed is equal to the stop value.
    // If LeftSpeed is greater than the stop value, it is decreased until the stop value is reached.
    // Beforehand, ForwardSpeed and ReverseSpeed are ensured to be reset to the stop value.
    if (SignalResults.value == 1400905448) {
      if (ForwardSpeed > StopValue | ReverseSpeed > StopValue) {
        ForwardSpeed = StopValue; ReverseSpeed = StopValue;       // Resetting ForwardSpeed and ReverseSpeed to the stop value
        ESC1.write(StopValue); ESC2.write(StopValue);             // and stopping the motors before proceeding
        ESC3.write(StopValue); ESC4.write(StopValue);             
        RightSpeed = StopValue - SpeedIncrement;                  // This ensures ESC2 and ESC3 are stopped until the
      }                                                           // right navigation button is pressed again.
      if (LeftSpeed > StopValue) {
        LeftSpeed = LeftSpeed - SpeedIncrement;                   // If Left is greater than StopValue, reduce speed
        ESC1.write(LeftSpeed); ESC4.write(LeftSpeed);             // to ESC1 and ESC4, otherwise, increase speed to ESC2 and ESC3
      }                                                           // in order to rotate the bot clockwise
      else {
        RightSpeed = RightSpeed + SpeedIncrement;
        if (RightSpeed > MaxSpeed) {RightSpeed = MaxSpeed;}       // The speed can be increased to the maximum allowable speed.
        ESC2.write(RightSpeed); ESC3.write(RightSpeed);
      }      
    }
    
    // Sending speed variable values to the serial monitor each time a button on the remote is pressed
    Serial.println(SignalResults.value);
    Serial.print("F: "); Serial.print(ForwardSpeed); Serial.print("  "); 
    Serial.print("B: "); Serial.println(ReverseSpeed);
    Serial.print("L: "); Serial.print(LeftSpeed); Serial.print("  ");
    Serial.print("R: "); Serial.println(RightSpeed);
    
    IRSignal.resume();
  }

   Measuring the battery level (%) at a specified time interval and sending information to the serial monitor
  CurrentTime = millis();
  if (CurrentTime - PreviousTime > TimeInterval) {
    PreviousTime = CurrentTime;
    AnalogIn = analogRead(BatteryPin);
    LipoLevel = (AnalogIn*0.0049/5)*100;
    Serial.print("Lipo Level: "); Serial.print(LipoLevel); Serial.println("%");
  }
}
